/*
 * Copyright (c) KATO Takenori, 1999.
 * 
 * All rights reserved.  Unpublished rights reserved under the copyright
 * laws of Japan.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * 	$Id: help.c,v 1.10 1999/06/08 10:02:38 kato Exp $
 */

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/Separator.h>

#include "keyboard.h"
#include "stack.h"
#include "option.h"

extern const char* version;
static Widget	aboutdialog, scdialog, scframe[NKEY];

void	DisplayAboutDialog(void);
void	DisplayShortcutDialog(void);
void	inithelp(Widget parent);

static void	CreateAboutDialogBox(Widget parent);
static void	CreateShortcutDialogBox(Widget parent);
void	SetShortcutChar(void);

void
DisplayAboutDialog(void)
{
	if (XtIsManaged(aboutdialog))
		return;
	XtManageChild(aboutdialog);
}

void
DisplayShortcutDialog(void)
{
	if (XtIsManaged(scdialog))
		return;

	XtManageChild(scdialog);
}

static void
CreateAboutDialogBox(Widget parent)
{
	Widget		label1, sep, ok;
	Arg			args[20];
	Cardinal	n;
	XmString	msg1, msg2;
	char		txt[1024];

	n = 0;
	aboutdialog = XmCreateFormDialog(parent, "aboutdialog", args, n);
	label1 = XtVaCreateManagedWidget("albl1",
									 xmLabelWidgetClass, aboutdialog,
									 XmNtopAttachment, XmATTACH_POSITION,
									 XmNtopPosition, 5,
									 XmNleftAttachment, XmATTACH_POSITION,
									 XmNleftPosition, 8,
									 XmNrightAttachment, XmATTACH_POSITION,
									 XmNrightPosition, 92,
									 XmNalignment, XmALIGNMENT_BEGINNING,
									 NULL);

	sprintf(txt, "   mrpnc ver. ");
	strcat(txt, version);
	strcat(txt, "\n");
	msg1 = XmStringCreateLtoR(txt, "LARGE");
	sprintf(txt,
			"    Motif(tm) RPN calculator\n"
			"    Copyright (C) 1999.  KATO Takenori\n"
			"    All rights reserved.\n\n"
			"    URL: http://ganko.eps.nagoya-u.ac.jp/~kato/mrpnc\n\n");
	msg2 = XmStringCreateLtoR(txt, "NORMAL");
	msg1 = XmStringConcat(msg1, msg2);
	XtVaSetValues(label1, XmNlabelString, msg1, NULL);
	XmStringFree(msg1);
	XmStringFree(msg2);

	sep = XtVaCreateManagedWidget("about_lbl1",
								  xmSeparatorWidgetClass, aboutdialog,
								  XmNtopAttachment, XmATTACH_WIDGET,
								  XmNtopWidget, label1,
								  XmNleftAttachment, XmATTACH_FORM,
								  XmNrightAttachment, XmATTACH_FORM,
								  NULL);
	ok = XtVaCreateManagedWidget("a_bt",
								 xmPushButtonWidgetClass, aboutdialog,
								 XmNtopAttachment, XmATTACH_WIDGET,
								 XmNtopWidget, sep,
								 XmNleftAttachment, XmATTACH_POSITION,
								 XmNleftPosition, 40,
								 XmNrightAttachment, XmATTACH_POSITION,
								 XmNrightPosition, 60,
								 NULL);
	msg1 = XmStringCreateSimple("Ok");
	XtVaSetValues(ok, XmNlabelString, msg1, NULL);
	XmStringFree(msg1);
}

static void
CreateShortcutDialogBox(Widget parent)
{
	Widget	keyform, frame, sep, ok;
	Arg	args[20];
	Cardinal	n;
	char	s[16];
	int	i;
	XmString	str;

	n = 0;
	scdialog = XmCreateFormDialog(parent, "shortcutdialog", args, n);

	keyform = XtVaCreateWidget("sckeyform",
	    xmFormWidgetClass, scdialog,
	    XmNtopAttachment, XmATTACH_POSITION,
	    XmNtopPosition, 10,
	    XmNleftAttachment, XmATTACH_POSITION,
	    XmNleftPosition, 10,
	    XmNrightAttachment, XmATTACH_POSITION,
	    XmNrightPosition, 90,
	    NULL);

	for (i = 0; i < NKEY; i++) {
		str = XmStringCreateSimple(keyboard_op->shortcutchar[i]);
		sprintf(s, "sckeyfr%d", i);
		frame = XtVaCreateManagedWidget(s,
		    xmFrameWidgetClass, keyform,
		    XmNtopAttachment, XmATTACH_POSITION,
		    XmNtopPosition, keypos[i].y1,
		    XmNleftAttachment, XmATTACH_POSITION,
		    XmNleftPosition, keypos[i].x1,
		    XmNrightAttachment, XmATTACH_POSITION,
		    XmNrightPosition, keypos[i].x2,
		    XmNbottomAttachment, XmATTACH_POSITION,
		    XmNbottomPosition, keypos[i].y2,
		    NULL);
		scframe[i] = XtVaCreateManagedWidget(s,
		    xmLabelWidgetClass, frame,
			XmNlabelString, str,
		    NULL);
		XmStringFree(str);
	}
	XtManageChild(keyform);

	sep = XtVaCreateManagedWidget("about_lbl1",
								  xmSeparatorWidgetClass, scdialog,
								  XmNtopAttachment, XmATTACH_WIDGET,
								  XmNtopWidget, keyform,
								  XmNleftAttachment, XmATTACH_FORM,
								  XmNrightAttachment, XmATTACH_FORM,
								  NULL);
	ok = XtVaCreateManagedWidget("a_bt",
								 xmPushButtonWidgetClass, scdialog,
								 XmNtopAttachment, XmATTACH_WIDGET,
								 XmNtopWidget, sep,
								 XmNleftAttachment, XmATTACH_POSITION,
								 XmNleftPosition, 40,
								 XmNrightAttachment, XmATTACH_POSITION,
								 XmNrightPosition, 60,
								 NULL);
	str = XmStringCreateSimple("Ok");
	XtVaSetValues(ok, XmNlabelString, str, NULL);
	XmStringFree(str);
}

void
inithelp(Widget parent)
{
	CreateAboutDialogBox(parent);
	CreateShortcutDialogBox(parent);
}

void
SetShortcutChar(void)
{
	int	i;
	XmString	str;

	for (i = 0; i < NKEY; i++) {
		str = XmStringCreateSimple(keyboard_op->shortcutchar[i]);
		XtVaSetValues(scframe[i], XmNlabelString, str, NULL);
		XmStringFree(str);
	}
}
