/*
 * Copyright (c) KATO Takenori, 1999.
 * 
 * All rights reserved.  Unpublished rights reserved under the copyright
 * laws of Japan.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * 	$Id: option.c,v 1.13 1999/06/08 11:55:36 kato Exp $
 */

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>

#include "stack.h"
#include "keyboard.h"
#include "compmode.h"
#include "basemode.h"
#include "console.h"
#include "help.h"
#include "option.h"

Widget	optionlabel;
struct stack_op_t	*stack_op;
struct keyboard_op_t	*keyboard_op;
int	angleunit = ANGLE_DEG;
int	mode = MODE_COMP;

static Widget	anglebt[NANGLE];
static Widget	createdeg(Widget parent);

struct mode_op_t {
	void	(*mode_stack)(struct stack_op_t**);
	void	(*mode_keyboard)(struct keyboard_op_t**);
	void	(*mode_keytop)(void);
	void	(*mode_optionlabel)(void);
};

static struct mode_op_t	mode_op[NMODE] = {
	{stack_compmode, keyboard_compmode, setkeytopall_comp,
	    optionlabel_compmode},
	{stack_basemode, keyboard_basemode, setkeytopall_base,
	    optionlabel_basemode}
};

void
setmode(int m)
{
	switch (m) {
	case MODE_COMP:
		if (mode != MODE_COMP) {
			stackGetX();
			mode = MODE_COMP;
			mode_op[mode].mode_stack(&stack_op);
			mode_op[mode].mode_keyboard(&keyboard_op);
			mode_op[mode].mode_keytop();
			mode_op[mode].mode_optionlabel();
			SetShortcutChar();
			displayall();
		}
		break;
	case MODE_BASE:
		if (mode != MODE_BASE) {
			stackGetX();
			mode = MODE_BASE;
			mode_op[mode].mode_stack(&stack_op);
			mode_op[mode].mode_keyboard(&keyboard_op);
			mode_op[mode].mode_keytop();
			mode_op[mode].mode_optionlabel();
			SetShortcutChar();
			displayall();
		}
		break;
	}
}

void
setangle(int a)
{
	switch (a) {
	case ANGLE_DEG:
		angleunit = ANGLE_DEG;
		XtVaSetValues(anglebt[ANGLE_DEG], XmNset, True, NULL);
		XtVaSetValues(anglebt[ANGLE_RAD], XmNset, False, NULL);
		XtVaSetValues(anglebt[ANGLE_GRAD], XmNset, False, NULL);
		break;
	case ANGLE_RAD:
		angleunit = ANGLE_RAD;
		XtVaSetValues(anglebt[ANGLE_RAD], XmNset, True, NULL);
		XtVaSetValues(anglebt[ANGLE_DEG], XmNset, False, NULL);
		XtVaSetValues(anglebt[ANGLE_GRAD], XmNset, False, NULL);
		break;
	case ANGLE_GRAD:
		angleunit = ANGLE_GRAD;
		XtVaSetValues(anglebt[ANGLE_GRAD], XmNset, True, NULL);
		XtVaSetValues(anglebt[ANGLE_RAD], XmNset, False, NULL);
		XtVaSetValues(anglebt[ANGLE_DEG], XmNset, False, NULL);
		break;
	}
}

static void
degCB(Widget p, XtPointer clientdata, XtPointer calldata)
{
	switch ((int)clientdata) {
	case ANGLE_DEG:
		angleunit = ANGLE_DEG;
		break;
	case ANGLE_RAD:
		angleunit = ANGLE_RAD;
		break;
	case ANGLE_GRAD:
		angleunit = ANGLE_GRAD;
		break;
	}
}

static Widget
createdeg(Widget parent)
{
	Widget	f, t, r, tg;

	f = XtVaCreateManagedWidget("degframe",
	    xmFrameWidgetClass, parent,
	    XmNtopAttachment, XmATTACH_FORM,
	    XmNleftAttachment, XmATTACH_FORM,
	    XmNrightAttachment, XmATTACH_FORM,
	    NULL);
#if (XmVersion > 1001)
	t = XtVaCreateManagedWidget("degtitle",
	    xmLabelWidgetClass, f,
	    XmNchildType, XmFRAME_TITLE_CHILD,
	    XmNchildHorizontalAlignment, XmALIGNMENT_BEGINNING,
	    XmNchildVerticalAlignment, XmALIGNMENT_BASELINE_BOTTOM,
	    NULL);
#endif

	r = XtVaCreateManagedWidget("degrowcol",
	    xmRowColumnWidgetClass, f,
	    XmNradioBehavior, True,
	    XmNradioAlwaysOne, True,
	    XmNmarginHeight, -0,
	    XmNspacing, -1,
	    NULL);

	tg = anglebt[ANGLE_DEG] = XtVaCreateManagedWidget("degree",
	    xmToggleButtonWidgetClass, r,
	    XmNindicatorType, XmONE_OF_MANY,
	    NULL);
	XtVaSetValues(tg,
	    XmNset, True,
	    NULL);
	XtAddCallback(tg, XmNvalueChangedCallback,
	    (XtCallbackProc)degCB, (XtPointer)ANGLE_DEG);

	tg = anglebt[ANGLE_RAD] = XtVaCreateManagedWidget("radian",
	    xmToggleButtonWidgetClass, r,
	    XmNindicatorType, XmONE_OF_MANY,
	    NULL);
	XtAddCallback(tg, XmNvalueChangedCallback,
	    (XtCallbackProc)degCB, (XtPointer)ANGLE_RAD);

	tg = anglebt[ANGLE_GRAD] = XtVaCreateManagedWidget("gradian",
	    xmToggleButtonWidgetClass, r,
	    XmNindicatorType, XmONE_OF_MANY,
	    NULL);
	XtAddCallback(tg, XmNvalueChangedCallback,
	    (XtCallbackProc)degCB, (XtPointer)ANGLE_GRAD);

	return f;
}

static Widget
createlabel(Widget parent)
{
	optionlabel = XtVaCreateWidget("optionlabel",
	    xmLabelWidgetClass, parent,
	    XmNtopAttachment, XmATTACH_FORM,
	    XmNleftAttachment, XmATTACH_FORM,
	    XmNrightAttachment, XmATTACH_FORM,
	    NULL);
	mode_op[mode].mode_optionlabel();
	XtManageChild(optionlabel);
	return optionlabel;
}

Widget
initoption(Widget parent)
{
	Widget degform, labelform;

	degform = XtVaCreateWidget("optionform",
	    xmFormWidgetClass, parent,
	    XmNtopAttachment, XmATTACH_FORM,
	    XmNleftAttachment, XmATTACH_POSITION,
	    XmNleftPosition, OPTIONFORM_LEFTPOS,
	    XmNrightAttachment, XmATTACH_FORM,
	    NULL);
	(void)createdeg(degform);
	XtManageChild(degform);
	labelform = XtVaCreateWidget("optionlabelform",
	    xmFormWidgetClass, parent,
	    XmNtopAttachment, XmATTACH_WIDGET,
	    XmNtopWidget, degform,
	    XmNleftAttachment, XmATTACH_POSITION,
	    XmNleftPosition, OPTIONFORM_LEFTPOS,
	    XmNrightAttachment, XmATTACH_FORM,
	    NULL);
	(void)createlabel(labelform);
	XtManageChild(labelform);

	return degform;
}

void
setstackop(void)
{
	mode_op[mode].mode_stack(&stack_op);
}

void
setkeyboardop(void)
{
	mode_op[mode].mode_keyboard(&keyboard_op);
}
