/*
 * Copyright (c) KATO Takenori, 1999.
 * 
 * All rights reserved.  Unpublished rights reserved under the copyright
 * laws of Japan.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * 	$Id: stack.c,v 1.15 1999/06/08 11:55:36 kato Exp $
 */

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/Separator.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "stack.h"
#include "keyboard.h"
#include "console.h"
#include "option.h"

int		isjustaftere = 0;

static Widget	stack[NSTACK];
static Widget	stacklabel[NSTACK];
static char		xchar[20];
static int		isexpmode = 0;
static int		stackXpos = 0;

static void	createstack(Widget parent);
static void	flushX(void);

static void
flushX(void)
{
	stackXpos = 0;
	isexpmode = 0;
	isjustaftere = 0;
	displayall();
}

void
displayall(void)
{
	XmString	label;
	char	val[128];
	int		i;

	for (i = 0; i < NSTACK; i++) {
		stack_op->printstack(val, i);
		label = XmStringCreateSimple(val);
		XtVaSetValues(stack[i],
		    XmNlabelString, label,
		    NULL);
		XmStringFree(label);
	}
}

static void
createstack(Widget parent)
{
	char	n[16];
	XmString	label;
	Widget	f, fm[NSTACK];
	int	i;

	for (i = NSTACK - 1; i >= 0; i--) {
		sprintf(n, "fm%02d", i);
		if (i == NSTACK - 1) {
			fm[i] = XtVaCreateWidget(n,
			    xmFormWidgetClass, parent,
			    XmNtopAttachment, XmATTACH_FORM,
			    XmNleftAttachment, XmATTACH_FORM,
			    XmNrightAttachment, XmATTACH_FORM,
			    NULL);
		} else {
			fm[i] = XtVaCreateWidget(n,
			    xmFormWidgetClass, parent,
			    XmNtopAttachment, XmATTACH_WIDGET,
			    XmNtopWidget, fm[i + 1],
			    XmNleftAttachment, XmATTACH_FORM,
			    XmNrightAttachment, XmATTACH_FORM,
			    NULL);
		}
		switch (i) {
		case NSTACK - 1:
			label = XmStringCreateSimple("T");
			sprintf(n, "labelT");
			break;
		case 0:
			label = XmStringCreateSimple("X");
			sprintf(n, "label%02d", i);
			break;
		case 1:
			label = XmStringCreateSimple("Y");
			sprintf(n, "label%02d", i);
			break;
		case 2:
			label = XmStringCreateSimple("Z");
			sprintf(n, "label%02d", i);
			break;
		default:
			sprintf(n, "S%d", i - 2);
			label = XmStringCreateSimple(n);
			sprintf(n, "label%02d", i);
			break;
		}
		stacklabel[i] = XtVaCreateManagedWidget(n,
		    xmLabelWidgetClass, fm[i],
		    XmNtopAttachment, XmATTACH_FORM,
		    XmNbottomAttachment, XmATTACH_FORM,
		    XmNleftAttachment, XmATTACH_FORM,
		    XmNrightAttachment, XmATTACH_POSITION,
		    XmNrightPosition, 10,
		    XmNlabelString, label,
		    NULL);
		XmStringFree(label);

		sprintf(n, "frame%02d", i);
		f = XtVaCreateManagedWidget(n,
		    xmFrameWidgetClass, fm[i],
		    XmNtopAttachment, XmATTACH_FORM,
		    XmNbottomAttachment, XmATTACH_FORM,
		    XmNleftAttachment, XmATTACH_POSITION,
		    XmNleftPosition, 12,
		    XmNrightAttachment, XmATTACH_FORM,
		    NULL);
		if (i == NSTACK - 1)
			sprintf(n, "stackT");
		else
			sprintf(n, "stack%02d", i);
		label = XmStringCreateSimple("");
		stack[i] = XtVaCreateManagedWidget(n,
		    xmLabelWidgetClass, f,
		    XmNlabelString, label,
		    XmNalignment, XmALIGNMENT_END,
		    XmNlayoutDirection, XmSTRING_DIRECTION_L_TO_R,
		    NULL);
		XmStringFree(label);
		XtManageChild(fm[i]);
	}
}

Widget
initstack(Widget parent)
{
	Widget	f;

	setstackop();

	stack_op->clearstackall();

	f = XtVaCreateWidget("stackform",
	    xmFormWidgetClass, parent,
	    XmNtopAttachment, XmATTACH_FORM,
	    XmNleftAttachment, XmATTACH_FORM,
	    XmNrightAttachment, XmATTACH_POSITION,
	    XmNrightPosition, STACKFORM_RIGHTPOS,
	    NULL);
	createstack(f);
	XtManageChild(f);
	xchar[0] = '\0';

	displayall();

	return f;
}

int
stackAddChar(char c)
{
	XmString	label;
	int	len;

	len = stackXpos;
	if (len == 19)
		return 1;

	if (len == 0) {
		stack_op->stackupward();
		displayall();
	}

	if (len == -1)
		stackXpos = len = 0;

	switch (c) {
	case '0':
		/* Don't allow starting with zero. */
		if (len == 1 && xchar[0] == '0')
			return 1;
		/* FALLTHROUGH */
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
		isjustaftere = 0;
		xchar[len] = c;
		stackXpos++;
		xchar[len+1] = '\0';
		break;
	case '.':
		if (isexpmode)
			return 1;
		xchar[len] = c;
		stackXpos++;
		xchar[len+1] = '\0';
		isjustaftere = 0;
		break;
	case 'a':
	case 'A':
	case 'b':
	case 'B':
	case 'c':
	case 'C':
	case 'd':
	case 'D':
	case 'E':
	case 'f':
	case 'F':
		isjustaftere = 0;
		xchar[len] = c;
		stackXpos++;
		xchar[len+1] = '\0';
		break;
	case 'e':
		if (mode == MODE_COMP) {
			if (len == 0)
				return 1;
			if (isexpmode)
				return 1;
			xchar[len] = c;
			stackXpos++;
			xchar[len+1] = '\0';
			isjustaftere = 1;
			isexpmode = 1;
		} else {
			isjustaftere = 0;
			xchar[len] = c;
			stackXpos++;
			xchar[len+1] = '\0';
		}
		break;
	case '+':
	case '-':
		if (!isexpmode || !isjustaftere)
			return 1;
		xchar[len] = c;
		stackXpos++;
		xchar[len+1] = '\0';
		isjustaftere = 0;
		break;
	}

	label = XmStringCreateSimple(xchar);
	XtVaSetValues(stack[0],
	    XmNlabelString, label,
	    NULL);
	XmStringFree(label);

	return 0;
}

int
stackBS(void)
{
	XmString	label;
	int	len;

	len = stackXpos;
	if (len == 0 || len == -1)
		return 0;
	if (isexpmode) {
		if (xchar[len - 2] == 'e')
			isjustaftere = 1;
	}
	if (xchar[len - 1] == 'e') {
		isexpmode = 0;
		isjustaftere = 0;
	}
	xchar[len - 1] = '\0';
	stackXpos--;
	label = XmStringCreateSimple(xchar);
	XtVaSetValues(stack[0],
	    XmNlabelString, label,
	    NULL);
	XmStringFree(label);

	return 0;
}

void
stackEnter(void)
{
	stack_op->stackenter(xchar, stackXpos);
	flushX();
	stackXpos = -1;
}

void
stackPush(void *d)
{
	stack_op->stackpush(d);
	flushX();
}

void*
stackPop(void)
{
	void*	X;

	X = stack_op->stackpop(xchar, stackXpos);
	flushX();

	return X;
}

void*
stackGetX(void)
{
	void*	dt;

	dt = stack_op->stackgetx(xchar, stackXpos);
	flushX();
	return dt;
}

void
stackSetX(void* d)
{
	stack_op->stacksetx(d);
	flushX();
}

void
stackSwap(void)
{
	stack_op->stackswap(xchar, stackXpos);
	flushX();
}

void
stackUp(void)
{
	stack_op->stackup(xchar, stackXpos);
	flushX();
}

void
stackDown(void)
{
	stack_op->stackdown();
	flushX();
}

void
stackSign(void)
{
	stack_op->stacksign(xchar, stackXpos);
	flushX();
}

void
stackClear(void)
{
	stack_op->stackclearx();

	flushX();
}

void
stackMem(void)
{
	stack_op->stackmem(xchar, stackXpos);
	flushX();
}

void
stackMemRead(void)
{
	stack_op->stackmemread();
	flushX();
}

void
stackMemAdd(void)
{
	stack_op->stackmemadd(xchar, stackXpos);
	flushX();
}

void
stackMemSub(void)
{
	stack_op->stackmemsub(xchar, stackXpos);
	flushX();
}

void
stackCopyY(void)
{
	stack_op->stackcopy_y();
	flushX();
}

void
stackCopyZ(void)
{
	stack_op->stackcopy_z();
	flushX();
}

void
stackCopyT(void)
{
	stack_op->stackcopy_t();
	flushX();
}
